import ctypes
import datetime
import sqlite3
import importlib
import tkinter as tk
import tkinter.ttk as ttk

from . import delete
from . import illegal
import nikkahyo
import value


def btn_click(root, radioValue, d_today, class_time, ids, name, button_size):
    """ ボタンをクリックしたとき.

    ボタンをクリックしたときの処理

    Args:
        root (instance): toplevel のインスタンス
        radioValue (int): ラジオボタンの値
        d_today (str): 日付
        class_time (int): 時間目
        ids (int): 生徒id
        name (str): 生徒名
        button_size (int): ボタンの幅の高さ
    """
    def x():
        illegal.window(
            root,
            radioValue,
            d_today,
            class_time,
            ids,
            name,
            button_size
        )
    return x


def btn_delete(root, d_today, class_time, classes, win_size):
    """ ボタンをクリックしたとき.

    ボタンをクリックしたときの処理

    Args:
        root (instance): toplevel のインスタンス
        radioValue (int): ラジオボタンの値
        d_today (str): 日付
        class_time (int): 時間目
        classes (int): 学年クラス
        win_size (int): ウインドウの幅と高さ
    """
    def x():
        delete.window(root, d_today, class_time, classes, win_size)
    return x


def get_year(month):
    d_today = datetime.date.today()
    if(d_today.month < 4):
        years = d_today.year-1
    else:
        years = d_today.year
    if(month.get() < 4):
        years = years+1
    return years


def root_ok(
        root,
        win_size,
        button_size,
        grade_no,
        classValue,
        timeValue,
        month,
        days,
        py_pass
):
    def x():
        root.destroy()
        class_time = timeValue.get()+1
        class_no = classValue.get()+1
        class_day = '{}-{}-{}'.format(get_year(month), month.get(), days.get())
        class_window_create(
            win_size,
            button_size,
            grade_no,
            class_no,
            class_day,
            class_time,
            py_pass
        )
    return x


def Cancel(root, py_pass):
    def x():
        root.destroy()
        nikkahyo.window(py_pass)
    return x


def window(py_pass, window=False, app=None):
    def x():
        importlib.reload(value)
        if window:
            app.destroy()
        grade_no = value.grade_no
        root = tk.Tk()
        classValue = tk.IntVar()
        timeValue = tk.IntVar()
        month = tk.IntVar()
        days = tk.IntVar()
        user32 = ctypes.windll.user32
        win_size = (
            root.winfo_screenwidth(),
            root.winfo_screenheight()-user32.GetSystemMetrics(4)
        )
        button_size = (win_size[0]/6, win_size[1]/8)
        root.title('日課表作成システム')
        root.resizable(False, False)
        root.geometry("%dx%d+0+0" % (win_size[0], win_size[1]))
        root.state('zoomed')
        val = ('１組', '２組', '３組', '４組', '５組', '６組', '７組', '８組')
        for i in range(value.max_class):
            radio = tk.Radiobutton(
                root,
                text=val[i],
                variable=classValue,
                anchor='w',
                value=i
            )
            radio.place(
                x=0+win_size[0]*0.05,
                y=button_size[1]*i,
                width=button_size[0],
                height=button_size[1]
            )
        val = ('１時間目', '２時間目', '３時間目', '４時間目', '５時間目', '６時間目', '７時間目', '８時間目')
        for i in range(6):
            radio = tk.Radiobutton(
                root,
                text=val[i],
                variable=timeValue,
                anchor='w',
                value=i
            )
            radio.place(
                x=button_size[0]+win_size[0]*0.05,
                y=button_size[1]*i,
                width=button_size[0],
                height=button_size[1]
            )
        text = tk.Text(root)
        text.place(
            x=button_size[0]*2+win_size[0]*0.05,
            y=button_size[1]*0,
            width=win_size[0]-(button_size[0]*2+win_size[0]*0.1),
            height=button_size[1]*6
        )
        val_month = (
            4, 5, 6, 7, 8, 9,
            10, 11, 12, 1, 2, 3)
        cb_month = ttk.Combobox(
                root,
                textvariable=month,
                values=val_month,
        )
        cb_month.place(
            x=button_size[0]*2+win_size[0]*0.05,
            y=button_size[1]*6,
            width=button_size[0]/2,
            height=button_size[1]
        )
        lbl = tk.Label(text='月')
        lbl.place(
            x=button_size[0]*2.5+win_size[0]*0.05+1,
            y=button_size[1]*6,
            width=50-2,
            height=button_size[1]
        )
        val_days = (
            1, 2, 3, 4, 5, 6, 7, 8, 9, 10,
            11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
            21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
            31)
        cb_days = ttk.Combobox(
                root,
                textvariable=days,
                values=val_days,
        )
        cb_days.place(
            x=button_size[0]*2+win_size[0]*0.05+button_size[0]/2+50,
            y=button_size[1]*6,
            width=button_size[0]/2,
            height=button_size[1]
        )
        lbl = tk.Label(text='日')
        lbl.place(
            x=button_size[0]*2.5+win_size[0]*0.05+button_size[0]/2+50,
            y=button_size[1]*6,
            width=50-2,
            height=button_size[1]
        )
        button = tk.Button(
                    root,
                    text='キャンセル',
                    command=Cancel(root, py_pass)
                )
        button.place(
            x=win_size[0]-button_size[0]*2,
            y=button_size[1]*7,
            width=button_size[0],
            height=button_size[1]
        )
        button = tk.Button(
                    root,
                    text='実行',
                    command=root_ok(
                        root,
                        win_size,
                        button_size,
                        grade_no,
                        classValue,
                        timeValue,
                        month,
                        days,
                        py_pass
                    )
                )
        button.place(
            x=win_size[0]-button_size[0],
            y=button_size[1]*7,
            width=button_size[0],
            height=button_size[1]
        )
        d_today = datetime.date.today()
        cb_month.set(d_today.month)
        cb_days.set(d_today.day)
        root.mainloop()
    return x


def class_window_create(
        win_size,
        button_size,
        grade_no,
        class_no,
        class_day,
        class_time,
        py_pass
):
    importlib.reload(value)
    root = tk.Tk()
    radioValue = tk.IntVar()
    root.title('日課表作成システム')
    root.resizable(False, False)
    root.geometry('{0}x{1}+0+0'.format(win_size[0], win_size[1]))
    root.state('zoomed')
    frame = tk.Frame(root, width=win_size[0], height=win_size[1])
    sql = {}
    conn = sqlite3.connect(value.database_name)
    c = conn.cursor()
    address = {}
    address[0] = grade_no*1000+class_no*100+1
    address[1] = address[0]+41
    c.execute('''SELECT seat, nt.id, ruby, name FROM ChooseTable AS ct
        LEFT OUTER JOIN NameTable AS nt
        ON nt.id == ct.id WHERE seat BETWEEN {0} AND {1}'''.format(
            address[0],
            address[1]
        )
    )
    for row in c.fetchall():
        sql[row[0]] = (row[1], row[2], row[3])
    conn.close()
    for i in range(6):
        for k in range(7):
            address = sql[grade_no*1000+class_no*100+6-k+7*i+1]
            if address[0]:
                state = tk.NORMAL
                text = '{0}\n{1}'.format(address[1], address[2])
                name = address[2]
                ids = address[0]
            else:
                state = tk.DISABLED
                text = ''
                name = ''
                ids = 0
            button = ttk.Button(
                frame,
                text=text,
                state=state,
                command=btn_click(
                    root,
                    radioValue,
                    class_day,
                    class_time,
                    ids,
                    name,
                    button_size
                )
            )
            button.place(
                x=+button_size[0]*i,
                y=button_size[1]*k,
                width=button_size[0],
                height=button_size[1]
            )
    button_delete = ttk.Button(
        frame,
        text='削除',
        command=btn_delete(
            root,
            class_day,
            class_time,
            grade_no*1000+class_no*100,
            win_size
        )
    )
    button_delete.place(
        x=button_size[0]*0,
        y=button_size[1]*7,
        width=button_size[0],
        height=button_size[1]
    )
    button_end = ttk.Button(
        frame,
        text='end',
        command=window(py_pass, True, root))
    button_end.place(
        x=button_size[0]*5,
        y=button_size[1]*7,
        width=button_size[0],
        height=button_size[1]
    )
    frame.pack(fill=tk.BOTH)
    root.mainloop()
