import math
import sqlite3
import importlib
import tkinter as tk
import tkinter.ttk as ttk

import value


class window():
    """ 削除ウインドウ.

    削除ウインドウのプログラム群

    Args:
        root (instance): toplevel のインスタンス
        radioValue (int): ラジオボタンの値
        class_day (str): 日付
        class_time (int): 時間目
        classes (int): 学年クラス
        win_size (list): ウインドウの幅と高さ
    """
    def __init__(self, root, class_day, class_time, classes, win_size):
        self.class_time = class_time
        self.class_day = class_day
        self.classes = classes
        delete_window_w = win_size[0]/2
        delete_window_h = win_size[1]/2
        text_w = delete_window_w/2
        num_button_w = text_w/4
        num_button_h = (delete_window_h-50)/4
        num = [7, 8, 9, 4, 5, 6, 1, 2, 3]
        self.delete_window = tk.Toplevel(
            root,
            width=delete_window_w,
            height=delete_window_h
        )
        self.delete_window.title('削除')
        self.tree_view = ttk.Treeview(self.delete_window)
        self.tree_view['columns'] = (1, 2, 3)
        self.tree_view['show'] = 'headings'
        self.tree_view.column(1, width=math.floor(text_w*0.2))
        self.tree_view.column(2, width=math.floor(text_w*0.4))
        self.tree_view.column(3, width=math.floor(text_w*0.4))
        self.tree_view.heading(1, text='ID')
        self.tree_view.heading(2, text='名前')
        self.tree_view.heading(3, text='項目')
        self.tree_view.place(
            x=0,
            y=0,
            width=text_w,
            height=delete_window_h
        )
        self.entry = tk.Entry(self.delete_window)
        self.entry.place(
            x=text_w,
            y=0,
            width=text_w,
            height=50
        )
        for i in range(9):
            button = tk.Button(
                self.delete_window,
                text=num[i],
                command=self.num_input(num[i])
            )
            button.place(
                x=text_w+num_button_w*(i % 3),
                y=50+num_button_h*(i//3),
                width=num_button_w,
                height=num_button_h
            )
        button = tk.Button(
            self.delete_window,
            text=0,
            command=self.num_input(0)
        )
        button.place(
            x=text_w+num_button_w*0,
            y=50+num_button_h*3,
            width=num_button_w*2,
            height=num_button_h
        )
        button_close = tk.Button(
            self.delete_window,
            text='閉じる',
            command=self.num_close
        )
        button_close.place(
            x=text_w+num_button_w*2,
            y=50+num_button_h*3,
            width=num_button_w*2,
            height=num_button_h
        )
        button_del = tk.Button(
            self.delete_window,
            text='←',
            command=self.num_del
        )
        button_del.place(
            x=text_w+num_button_w*3,
            y=50+num_button_h*0,
            width=num_button_w,
            height=num_button_h
        )
        button_ok = tk.Button(
            self.delete_window,
            text='実\n行',
            command=self.num_ok
        )
        button_ok.place(
            x=text_w+num_button_w*3,
            y=50+num_button_h*1,
            width=num_button_w,
            height=num_button_h*2
        )
        self.tree_view_insert()
        self.tree_view.bind('<Double-1>', self.on_double_click)
        self.entry.focus_set()

    def num_input(self, i):
        """ 数字ボタンが押されたとき.

        数字ボタンが押されたときに文字を入力する

        Args:
            i (int): 押された数字ボタン
        """
        def x():
            self.entry.insert(tk.END, str(i))
            self.entry.focus_set()
        return x

    def num_del(self):
        """ ←ボタンが押されたとき.

        ←ボタンが押されたときに文字を削除する
        """
        num = self.entry.get()
        self.entry.delete(len(num)-1, tk.END)
        self.entry.focus_set()

    def num_ok(self):
        """ 実行ボタンが押されたとき.

        実行ボタンが押されたときに削除する
        """
        importlib.reload(value)
        conn = sqlite3.connect(value.database_name)
        c = conn.cursor()
        c.execute('''DELETE FROM DataTable
            WHERE list={0}'''.format(self.entry.get()))
        conn.commit()
        conn.close()
        self.entry.delete(0, tk.END)
        for i in self.tree_view.get_children():
            self.tree_view.delete(i)
        self.tree_view_insert()
        self.entry.focus_set()

    def num_close(self):
        """ 閉じるボタンが押されたとき.

        閉じるボタンが押されたときにウインドウを閉じる
        """
        self.delete_window.destroy()

    def tree_view_insert(self):
        """ ツリービューに挿入.

        ツリービューに授業の不正行為を入力する
        """
        importlib.reload(value)
        conn = sqlite3.connect(value.database_name)
        c = conn.cursor()
        c.execute('''SELECT * FROM DataTable AS dt
            INNER JOIN NameTable AS nt
            ON dt.ID==nt.ID
            WHERE class_time={0}
            AND date='{1}'
            AND dt.id BETWEEN {2} AND {3}
            '''.format(
                self.class_time,
                self.class_day,
                self.classes,
                self.classes+45
            )
        )
        for row in c.fetchall():
            for i in range(len(value.illegal)):
                if row[i+4]:
                    buf = value.illegal[i]
                    break
            self.tree_view.insert(
                '',
                'end',
                values=(row[0], row[len(value.illegal)+5], buf)
            )
        conn.close()

    def on_double_click(self, event):
        """ ツリービューがダブルクリックされたとき.

        ツリービューがダブルクリックされたときに文字を入力する

        Args:
            event (instance): tkinter.Event のインスタンス
        """
        curItem = self.tree_view.focus()
        buf = self.tree_view.item(curItem)['values']
        self.entry.delete(0, tk.END)
        self.entry.insert(tk.END, str(buf[0]))
        self.entry.focus_set()
